from datetime import datetime
from lib.trust.transaction import Transaction
from typing import Dict


class TrustTrace(Transaction):
    def __init__(self, use_case: str, context: str, user: str, table: str, volume: float, timestamp: datetime, is_trustworthy: float = 1):
        super().__init__(timestamp, volume, is_trustworthy=is_trustworthy)

        self.use_case = use_case
        self.context = context
        self.user = user
        self.table = table

    def to_serializable_dict(self):
        return {
            "volume": self.volume,
            "trustworthy": self.trustworthy,
            "timestamp": self.timestamp.isoformat(),
            "use_case": self.use_case,
            "context": self.context,
            "user": self.user,
            "table": self.table,
        }

    @staticmethod
    def from_serializable_dict(data: Dict):
        if "volume" not in data.keys() or "trustworthy" not in data.keys() or "timestamp" not in data.keys() or "use_case" not in data.keys() or "context" not in data.keys() or "user" not in data.keys() or "table" not in data.keys():
            raise ValueError("Missing fields for User!")

        return TrustTrace(
            data["use_case"],
            data["context"],
            data["table"],
            data["volume"],
            datetime.fromisoformat(data["timestamp"]),
            is_trustworthy=data["trustworthy"]
        )
