# global imports (dont't worry, red is normal)
import network_constants as netconst
from database.MongoRepositoryBase import MongoRepositoryBase

from lib.database.entities.trust_adapter import TrustAdapter
from typing import List, Dict

class TrustAdapterRepository(MongoRepositoryBase):
    '''This is a repository for MongoDb.'''

    def __init__(self):
        super().__init__(netconst.REPUTATION_CALCULATION_DB_HOSTNAME,
                         netconst.REPUTATION_CALCULATION_DB_PORT,
                         'reputation-computation-db')
        self._collection = 'trust_adapter'

    def add(self, adapter: TrustAdapter):
        super().insert_entry(self._collection, adapter.to_serializable_dict())

    def all(self) -> List[Dict]:
        result = super().get_entries(self._collection, projection={'_id': False})

        return list(result)

    def all_for_use_case_and_context(self, use_case: str, context: str) -> List[Dict]:
        result = super().get_entries(
            self._collection, 
            projection={'_id': False}, 
            selection={"use_case": use_case, "context": context}
        )

        return list(result)
    
    def all_for_use_case_and_table(self, use_case: str, table: str) -> List[Dict]:
        result = super().get_entries(
            self._collection, 
            projection={'_id': False}, 
            selection={"use_case": use_case, "table": table}
        )

        return list(result)

    def one_for_use_case_and_context_and_table(self, use_case: str, context: str, table: str) -> Dict:
        result = super().get_entries(
            self._collection, 
            projection={'_id': False}, 
            selection={"use_case": use_case, "context": context, "table": table}
        )

        result = list(result)

        if len(result) != 1:
            return None

        return result[0]

        