# global imports (dont't worry, red is normal)
import network_constants as netconst
from database.MongoRepositoryBase import MongoRepositoryBase

from lib.database.entities.user import User
from lib.database.entities.user_trust import UserTrust
from typing import List, Dict


class UserTrustRepository(MongoRepositoryBase):
    '''This is a repository for MongoDb.'''

    def __init__(self):
        super().__init__(netconst.REPUTATION_CALCULATION_DB_HOSTNAME,
                         netconst.REPUTATION_CALCULATION_DB_PORT,
                         'reputation-computation-db')
        self._collection = 'user_trust'

    def add(self, user_trust: UserTrust):
        super().insert_entry(self._collection, user_trust.to_serializable_dict())

    def all(self) -> List[UserTrust]:
        result = super().get_entries(
            self._collection, projection={'_id': False})

        return [UserTrust.from_serializable_dict(row) for row in list(result)]

    def all_for_use_case(self, use_case: str) -> List[UserTrust]:
        result = super().get_entries(self._collection, projection={
            '_id': False}, selection={"use_case": use_case})

        return [UserTrust.from_serializable_dict(row) for row in list(result)]

    def all_for_user(self, user: User) -> List[UserTrust]:
        result = super().get_entries(self._collection, projection={
            '_id': False}, selection={"use_case": user.use_case, "user": user.name})

        return [UserTrust.from_serializable_dict(row) for row in list(result)]

    def one_for_user_and_context(self, user: User, context: str) -> UserTrust:
        result = super().get_entries(self._collection, projection={'_id': False}, selection={
            "use_case": user.use_case, "user": user.name, "context": context})

        result = list(result)

        if len(result) != 1:
            return None

        return UserTrust.from_serializable_dict(result[0])

    def update(self, user_trust: UserTrust):
        collection = self._database[self._collection]
        collection.update_one({"use_case": user_trust.use_case, "context": user_trust.context, "user": user_trust.user}, {
                              "$set": user_trust.to_serializable_dict()})
