from lib.database.repositories.context_repository import ContextRepository
from lib.database.entities.context import Context
from lib.database.entities.trust_adapter import TrustAdapter
from lib.database.repositories.context_repository import ContextRepository
from lib.database.repositories.trust_adapter_repository import TrustAdapterRepository

from flask import request, Response
from typing import List

context_repository: ContextRepository = ContextRepository()
trust_adapter_repository: TrustAdapterRepository = TrustAdapterRepository()


def add(use_case: str, context: str):
    _ensure_context_exists_for_use_case(use_case, context)

    data = request.json

    adapter_new: TrustAdapter = None
    try:
        adapter_new = TrustAdapter.from_serializable_dict(data)
    except ValueError:
        return Response(status=400, response="Missing fields in request")

    if adapter_new.context != context:
        return Response(status=400, response="Context in body and url do not match!")

    if adapter_new.use_case != use_case:
        return Response(status=400, response="Use-Cases in body and url do not match!")

    adapter_reference = trust_adapter_repository.one_for_use_case_and_context_and_table(
        use_case, context, adapter_new.table)
    if adapter_reference != None:
        return Response(status=400, response="There already exists a TrustAdapter for this use-case/context/table")

    trust_adapter_repository.add(adapter_new)

    return adapter_new.to_serializable_dict()


def all_for_use_case_and_context(use_case: str, context: str):
    _ensure_context_exists_for_use_case(use_case, context)

    return trust_adapter_repository.all_for_use_case_and_context(use_case, context)


def _ensure_context_exists_for_use_case(use_case: str, context: str) -> Context:
    contexts: List[Context] = context_repository.all_for_use_case(use_case)
    target_context = None
    for c in contexts:
        if c["name"] == context:
            target_context = c
            break
    if target_context == None:
        return Response(status=404, response=f"Context {context} not found!")

    return Context.from_serializable_dict(target_context)
